/* bttv.h - interface for handling v4l and lowlevel bttv input
 *
 * Memory map of internal BTTV registers (C) Ewald Snel 1998-1999
 * (copied from the GPL'd program cabletv-1.3.9)
 * The rest: Copyright (C) 2003 Rik Snel <snel@phys.uu.nl>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; only version 2 of the License, not
 * any other version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 
 *
 * $Id: bttv.h 59 2004-07-30 21:47:15Z rsnel $
 */
#ifndef INCLUDE_CC2Y4M_BTTV_H
#define INCLUDE_CC2Y4M_BTTV_H 1

#include <linux/videodev.h>
#include <pthread.h>

#define BT848	0x0350109e	/* Brooktree BT848 VIDCAP */
#define BT849	0x0351109e	/* Brooktree BT849 VIDCAP */
#define BT878	0x036e109e	/* Brooktree BT878 VIDCAP */
#define BT879	0x036f109e	/* Brooktree BT879 VIDCAP */

/*
 * Local registers memory map for BT848 - BT879A
 * Author :  Ewald Snel, 1998 - 1999.
 * -description of local registers memory map
 * (may give endianness trouble? 
 */
typedef volatile struct {
	unsigned int		/* Device Status Register (DSTATUS) */
		cof:1,
		lof:1,
		plock:1,	/* unused */
		csel:1,
		numl:1,
		evenfield:1,
		hloc:1,
		pres:1, :24;
	unsigned int		/* Input Format Register (IFORM) */
		format:3,
		xtsel:2,
		muxsel:2,
		hactive:1, :24;
	unsigned int		/* Temporal Decimation Register (TDEC) */
		dec_rate:6,
		dec_fieldalign:1,
		dec_field:1, :24;
	unsigned int		/* MSB Cropping Register (E_CROP) */
		e_hactive_msb:2,
		e_hdelay_msb:2,
		e_vactive_msb:2,
		o_vdelay_msb:2,:24;
	unsigned int		/* Vertical Delay Register, Lower Byte */
		o_vdelay_lo:8, :24;
	unsigned int		/* Vertical Active Register, Lower Byte */
		e_vactive_lo:8, :24;
	unsigned int		/* Horizontal Delay Register, Lower Byte */
		e_hdelay_lo:8, :24;
	unsigned int		/* Horizontal Active Register, Lower Byte */
		e_hactive_lo:8, :24;
	signed int		/* Horizontal Scaling Register, Upper Byte */
		e_hscale_hi:8, :24;
	unsigned int		/* Horizontal Scaling Register, Lower Byte */
		e_hscale_lo:8, :24;
	signed int		/* Brightness Control Register */
		bright:8, :24;
	unsigned int		/* Miscellaneous Control Register (E_CONTROL) */
		e_sat_v_hi:1,
		e_sat_u_hi:1,
		e_contrast_hi:1,
		:1,		/* (0) res */
		e_cbsense:1,
		e_ldec:1,
		e_comp:1,
		e_lnotch:1, :24;
	unsigned int		/* Luma Gain Register, Lower Byte */
		contrast_lo:8, :24;
	unsigned int		/* Chroma (U) Gain Register, Lower Byte */
		sat_u_lo:8, :24;
	unsigned int		/* Chroma (V) Gain Register, Lower Byte */
		sat_v_lo:8, :24;
	signed int		/* Hue Control Register */
		hue:8, :24;
	unsigned int		/* SC Loop Control Register */
		:3,		/* (000) res */
		e_hfilt:2,
		e_ckill:1,
		e_cagc:1,
		e_peak:1, :24;
	unsigned int		/* White Crush Up Register  (WC_UP) */
		UPCNT:6,	/* unused */
		MAJS:2,		/* unused */
		:24;
	unsigned int		/* Output Format Register  (OFORM) */
		:5,		/* (00000) res */
		core:2,
		range:1, :24;
	unsigned int		/* VScale Register, Upper Byte (E_VSCALE_HI) */
		e_vscale_hi:5,
		e_uint:1,	/* unused */
		e_comb:1,
		e_vsfldalign:1,	/* unused */
		:24;
	unsigned int		/* Vertical Scaling Register, Lower Byte */
		e_vscale_lo:8, :24;
	unsigned int		/* Test Control Register */
		test:8, :24;
	int reg58:8, :24;	/* (?) */
	int reg5C:8, :24;
	unsigned int		/* AGC Delay Register */
		adelay:8, :24;
	unsigned int		/* Burst Delay Register */
		bdelay:8, :24;
	unsigned int		/* ADC Interface Register */
		crush:1,
		c_sleep:1,
		y_sleep:1,
		clk_sleep:1,
		agc_en:1,
		sync_t:1,
		:2, :24;	/* (10) res */
	unsigned int		/* Video Timing Control Register  (E_VTC) */
		e_vfilt:3,
		:4,		/* (0000) res */
		e_hsfmt:1,	/* unused */
		:24;
	int reg70:32;		/* (?) */
	int reg74:32;		/* (?) */
	unsigned int		/* White Crush Down Register */
		DNCNT:6,	/* unused */
		WCFRAME:1,	/* unused */
		VERTEN:1,	/* unused */
		:24;
	unsigned int		/* Software Reset Register */
		sreset:8, :24;
	unsigned int		/* Timing Generator Load Byte */
		tglb:8,		/* nused */
		:24;
	unsigned int		/* Timing Generator Control  (TGCTRL) */
		tgc_vm:1,	/* unused */
		gpc_ar:1,	/* unused */
		tgc_ai:1,	/* unused */
		tgcki:2,	/* unused */
		tgcko:2,	/* unused */
		:1, :24;	/* (0) res */
	int reg88:8, :24;
	unsigned int		/* MSB Cropping Register  (O_CROP) */
		o_hactive_msb:2,
		o_hdelay_msb:2,
		o_vactive_msb:2,
		e_vdelay_msb:2,:24;
	unsigned int		/* VDelay Reg, Lower Byte (E_VDELAY_LO) */
		e_vdelay_lo:8, :24;
	unsigned int		/* VActive Reg, Lower Byte (O_VACTIVE_LO) */
		o_vactive_lo:8, :24;
	unsigned int		/* HDelay Reg, Lower Byte (O_HDELAY_LO) */
		o_hdelay_lo:8, :24;
	unsigned int		/* HActive Reg, Lower Byte (O_HACTIVE_LO) */
		o_hactive_lo:8, :24;
	signed int		/* HScale Register, Upper Byte (O_HSCALE_HI) */
		o_hscale_hi:8, :24;
	unsigned int		/* HScale Register, Lower Byte (O_HSCALE_LO) */
		o_hscale_lo:8, :24;
	int regA8:8, :24;
	unsigned int		/* Miscellaneous Control Register (E_CONTROL) */
		o_sat_v_hi:1,
		o_sat_u_hi:1,
		o_contrast_hi:1,
		:1,		/* (0) res */
		o_cbsense:1,
		o_ldec:1,
		o_comp:1,
		o_lnotch:1, :24;
	unsigned int		/* Total Line Count Register */
		vtotal_lo:8, :24;
	unsigned int		/* Total Line Count Register */
		vtotal_hi:2,
		:6, :24;	/* (000000) res */
	int regB8:8, :24;
	int regBC:8, :24;
	unsigned int		/* SC Loop Control Register (O_SCLOOP) */
		:3,		/* (000) res */
		o_hfilt:2,
		o_ckill:1,
		o_cagc:1,
		o_peak:1, :24;
	int regC4:8, :24;
	int regC8:8, :24;
	unsigned int		/* VScale Register, Upper Byte (O_VSCALE_HI) */
		o_vscale_hi:5,
		o_uint:1,	/* unused */
		o_comb:1,
		o_vsfldalign:1,/* unused */
		:24;
	unsigned int		/* Vertical Scaling Register, Lower Byte */
		o_vscale_lo:8, :24;
	unsigned int		/* Color Format Register  (COLOR_FMT) */
		color_even:4,
		color_odd:4, :24;
	unsigned int		/* Color Control Register  (COLOR_CTL) */
		bswap_even:1,
		bswap_odd:1,
		wswap_even:1,
		wswap_odd:1,
		gamma:1,
		rgb_ded:1,	/* unused */
		color_bars:1,
		ext_frmrate:1,	/* unused */
		:24;
	unsigned int		/* Capture Control Register (CAP_CTL) */
		capture_even:1,
		capture_odd:1,
		capture_vbi_even:1,
		capture_vbi_odd:1,
		dith_frame:1,	/* unused */
		:3, :24;	/* (000) res */
	unsigned int		/* VBI Packet Size Register (VBI_PACK_SIZE) */
		vbi_pkt_lo:8,	/* unused */
		:24;
	unsigned int		/* VBI Packet Size / Delay Reg (VBI_PACK_DEL) */
		vbi_pkt_hi:1,	/* unused */
		ext_frame:1,
		vbi_hdelay:6,	/* unused */
		:24;
	unsigned int		/* Field Capture Counter Register (FCAP) */
		fcntr:8,
		:24;
	unsigned int		/* Video Timing Control Register (O_VTC) */
		o_vfilt:3,
		:4,		/* (0000) res */
		o_hsfmt:1,	/* unused */
		:24;
	unsigned int		/* PLL Reference Multiplier */
		pll_f_lo:8, :24;
	unsigned int		/* PLL Reference Multiplier */
		pll_f_hi:8, :24;
	unsigned int		/* Integer PLL-XCI Register */
		pll_i:6,
		pll_c:1,
		pll_x:1, :24;
	unsigned int		/* Digital Signal Interface Register (DVSIF) */
		dvsif:8, :24;
	unsigned int		/* Interrupt Status Register (INT_STAT) */
		interruptStatus:32;
	unsigned int		/* Interrupt Status Register (INT_MASK) */
		interruptMask:32;
	int reg108:8,		/* (?) */
		:24;
	unsigned int		/* GPIO and DMA Control Reg (GPIO_DMA_CTL) */
		fifo_enable:1,
		risc_enable:1,
		pktp:2,		/* unused */
		pltp1:2,	/* unused */
		pltp23:2,	/* unused */
		:2,		/* (00) res */
		gpclkmode:1,	/* unused */
		gpiomode:2,	/* unused */
		gpwec:1,	/* unused */
		gpuinti:1,	/* unused */
		gpuintc:1,	/* unused */
		:16;
	struct {
    		unsigned int sda:1, scl:1, w3b:1, sync:1, div:4, 
				db2:8, db1:8, db0:8;
	} I2C;
	unsigned int		/* RISC Program Start Address (RISC_STRT_ADD) */
		risc_ipc:32;
	unsigned int		/* GPIO Output Enable Control (GPIO_OUT_EN) */
		gpoe:24, :8;
	unsigned int		/* GPIO Input Control  (GPIO_REG_INP) */
		GPIE:24,	/* unused */
		:8;
	unsigned int		/* RISC Program Counter Register */
		risc_pc:32;

/* -----------------------------------------------// */
	int :32, :32, :32, :32, :32, :32, :32;
	int :32, :32, :32, :32, :32, :32, :32, :32;
	int :32, :32, :32, :32, :32, :32, :32, :32;
	int :32, :32, :32, :32, :32, :32, :32, :32;
	int :32, :32, :32, :32, :32, :32, :32, :32;
	int :32, :32, :32, :32, :32, :32, :32, :32;
	int :32, :32, :32, :32, :32, :32, :32, :32;
/* -----------------------------------------------// */

	unsigned int		/* GPIO Data I/O  (GPIO_DATA) */
		GPData:32;
} local_regs_t;

typedef struct {
	double pll;
	int vtotal;
} bttv_finfo_t;

typedef struct {
	int fd;
	int memfd;

	struct video_capability vc;
	struct video_channel *channels;
	struct video_mbuf buf_info;
	char *map;
	struct video_mmap maps[4];

	int type;
	
	int hdelay, hactive, field_height;
	
	/* buffer management */
	pthread_mutex_t bmutex;
	int synced, queued0, queued1, exported, avail;
	pthread_cond_t acond;
	bttv_finfo_t finfos[10];
	int framenoes[4];

	local_regs_t *regs;

	/* commands */
	pthread_mutex_t cmutex;
	int updown;
	double newfrq;

	/* info to restore bttv to old self */
	double old_pll;
	int old_vtotal;
	__s32 old_automute;
	local_regs_t old_regs;
} bttv_t;

int bttv_init(bttv_t*, char*, long chan);

double bttv_set_pll(bttv_t*, double);

double bttv_get_pll(bttv_t*);

void bttv_set_vtotal(bttv_t*, int);

int bttv_get_vtotal(bttv_t*);

char *bttv_getnext(bttv_t*, int*);

void bttv_capture_thread(void*);

void bttv_uninit(bttv_t*);

void bttv_pll_up(bttv_t*);

void bttv_pll_down(bttv_t*);

void bttv_schedule_updown(bttv_t*, int);

void bttv_schedule_pll(bttv_t*, double);

void bttv_set_frequency(bttv_t *bttv,long chan);

#endif /* INCLUDE_CC2Y4M_BTTV_H */
