/*
        This file is based on modes.y from the MPlayer project
	Modifications (C) 2004 RB

	The original copyright of the file follows below
	$Id: config.y 59 2004-07-30 21:47:15Z rsnel $
*/

/*
 *  Linux Frame Buffer Device Configuration
 *
 *   Copyright 1995-1998 by Geert Uytterhoeven
 *                     (Geert.Uytterhoeven@cs.kuleuven.ac.be)
 *
 *  --------------------------------------------------------------------------
 *
 *  This file is subject to the terms and conditions of the GNU General Public
 *  License. See the file COPYING in the main directory of the Linux
 *  distribution for more details.
 */ 
	   

%{

#define YYSTYPE		long

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "cc2y4m.h"
#include "parser.h"

extern int yylex(void);
extern void yyerror(const char *s);
extern int line;
extern struct lock_poldetect global_par;
struct channel_s chan={NULL,-2,{-1,-1,-1,-1,-1,-1,-1,NULL,-1},NULL,NULL};
double dbl;

void clean_chan()
{
	chan.name=NULL;
	chan.freq=-2;
	chan.p.polx=-1;
	chan.p.poly=-1;
	chan.p.poltresh=-1;
	chan.p.filter=-1;
	chan.p.polnorm=-1;
	chan.p.bright_adjust=-1;
	chan.p.center=-1;
	chan.p.refpoint=NULL;
	chan.p.locktresh=-1;
}
%}

%start init

%token POLX POLY POLTRESH POLNORM BRIGHT_ADJUST CENTER REFPOINT LOCKTRESH COMPOSITE
       CHAN FREQ GLOBAL CHANNEL M1 M2 INTEGER FILTER STRING DOUBLE REFVAL END

%%

init	  : globals
	  ;


globals	  : /* empty */
	  | global channels channel
	  ;


global	  : GLOBAL goptions END
	  ;

goptions  : /* empty */
	  | goptions gpolx
	  | goptions gpoly
	  | goptions gpoltresh
	  | goptions gpolnorm
	  | goptions gbright
	  | goptions gcenter
	  | goptions grefpoint
	  | goptions glocktresh
	  | goptions gfilter
	  ;

gpolx	  : POLX INTEGER
	    {
		global_par.polx = $2;
	    }
	  ;

gpoly	  : POLY INTEGER
	    {
		global_par.poly = $2;
	    }
	  ;

gpoltresh  : POLTRESH INTEGER
	    {
		global_par.poltresh = $2;
	    }
	  ;

gpolnorm	  : POLNORM INTEGER
	    {
		if( $2 != 0 && $2 != 1)
			parse_error("polnorm must be 0 or 1");
		global_par.polnorm = $2;
	    }
	  ;

gfilter 	  : FILTER INTEGER
	    {
		if( $2 != 0 && $2 != 1)
			parse_error("filter must be 0 or 1");
		global_par.filter = $2;
	    }
	  ;

gbright	  : BRIGHT_ADJUST INTEGER
	    {
		global_par.bright_adjust = $2;
	    }
	  ;

gcenter	  : CENTER INTEGER
	    {
		global_par.center = $2;
	    }
	  ;

grefpoint  : REFPOINT REFVAL
	    {
		if($2 == M1) {
			global_par.refpoint = refpoint_m1;
		}
		else if($2 == M2) {
			global_par.refpoint = refpoint_m2;
		}
	    }
	  ;

glocktresh : LOCKTRESH INTEGER
	    {
		global_par.locktresh = $2;
	    }
	  ;

channels  :
	  | channels channel
	  ;

channel	  : CHANNEL STRING coptions END
	    {
		chan.name=strdup((const char *)$2);
		new_channel(&chan);
		clean_chan();
	    }
	  ;

coptions  : /* empty */
	  | coptions cpolx
	  | coptions cpoly
	  | coptions cpoltresh
	  | coptions cpolnorm
	  | coptions cbright
	  | coptions ccenter
	  | coptions crefpoint
	  | coptions clocktresh
	  | coptions cchan
	  | coptions cfreq
	  | coptions cfilter
	  | coptions ccomposite
	  ;

cpolx	  : POLX INTEGER
	    {
		chan.p.polx=$2;
	    }
	  ;

cpoly	  : POLY INTEGER
	    {
		chan.p.poly=$2;
	    }
	  ;

cpoltresh  : POLTRESH INTEGER
	    {
		chan.p.poltresh=$2;
	    }
	  ;

cpolnorm	  : POLNORM INTEGER
	    {
		if( $2 != 0 && $2 != 1)
			parse_error("polnorm must be 0 or 1");
		chan.p.polnorm=$2;
	    }
	  ;

cbright	  : BRIGHT_ADJUST INTEGER
	    {
		chan.p.bright_adjust=$2;
	    }
	  ;

cfilter 	  : FILTER INTEGER
	    {
		if( $2 != 0 && $2 != 1)
			parse_error("filter must be 0 or 1");
		global_par.filter = $2;
	    }
	  ;

ccenter	  : CENTER INTEGER
	    {
		chan.p.center=$2;
	    }
	  ;

crefpoint  : REFPOINT REFVAL
	    {
		if($2 == M1) 
			chan.p.refpoint=refpoint_m1;
		else if($2 == M2) 
			chan.p.refpoint=refpoint_m2;
	    }
	  ;

clocktresh : LOCKTRESH INTEGER
	    {
		chan.p.locktresh=$2;
	    }
	  ;

cchan	   : CHAN STRING
	    {
		dbl=lookup_frequency((char *)$2);
                if(dbl==0.0)
                        parse_error("Uknown channel %s",(char *)$2);
                chan.freq=(long)(1000000*(dbl/62.5));
	    }
	  ;

cfreq	   : FREQ DOUBLE
	    {
		chan.freq=(long)(1000000*((*((double *)$2))/62.5));
	    }
	  ;

ccomposite : COMPOSITE
	   {
	   	chan.freq = -1;
	   }
	;
%%

