/* mplayerkeys.c - input keys from MPlayer window
 *
 * Copyright (C) 2004 RB
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; only version 2 of the License, not
 * any other version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 
 *
 * $Id: mplayerKeys.c 59 2004-07-30 21:47:15Z rsnel $
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <signal.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/keysym.h>

#include "cc2y4m.h"
#include "mplayerKeys.h"

/* The code for find_window was taken from the lirc project
   written by Heinrich Langos  <heinrich@null.net>
*/

Window find_window(Window,char *);
int xerrhnd(Display *, XErrorEvent *);


static Display *dpy=NULL;
static Window w=0;
static XKeyEvent *kev;


static Window root;
static time_t time_now;
static int still_looking=1;

char *lasttitle;

void titlehnd(int sig)
{
	mplayer_set_title(lasttitle);
}	

void init_mplayer_keys()
{
	dpy=XOpenDisplay(NULL);
	if(dpy==NULL) {
        	fprintf(stderr,"Could't open the display\n");
        	exit(1);
  	}
	
	root=RootWindow(dpy,DefaultScreen(dpy));
	XSetErrorHandler(xerrhnd);
	
	time_now=time(NULL);
	signal(SIGALRM,titlehnd);
}

char *get_mplayer_keys()
{
	XEvent event;
	int nbytes;
	char str[256],*ksname=NULL;
	KeySym ks;

	if(still_looking) {
		w=find_window(root,"MPlayer");
		if(w) {
			XSelectInput(dpy, w, KeyPressMask);
			still_looking=0;
		}
		if(time(NULL)-time_now>5) {
			VERBOSE("\nCould not find the MPlayer window. "
				"Giving up\n");
			still_looking=0;
			return NULL;
		}
	}

	if(!w) return NULL;

	if(XCheckWindowEvent(dpy,w,KeyPressMask,&event)) {
		kev=(XKeyEvent *)&event;
		nbytes = XLookupString (kev, str, 256, &ks, NULL);
		if(ks==NoSymbol) return NULL;
		ksname = XKeysymToString (ks);
	}
	return ksname;
	
}

void mplayer_set_title(char *s)
{
	lasttitle=s;
	if(still_looking) {
		alarm(1);
		return;
	}
	XStoreName(dpy,w,s);
}

Window find_window(Window top,char *name)
{
  char *wname,*iname;
  XClassHint xch;
  Window *children,foo;
  unsigned int nc;

  /* First the base case */
  if (XFetchName(dpy,top,&wname)){
    if (!strncmp(wname,name,strlen(name)))  {
      XFree(wname);
      VERBOSE("found %s by wname %x \n",name,(unsigned int)top);
      return(top);  /* found it! */
    }
    XFree(wname);
  }


  if(XGetIconName(dpy,top,&iname)){
    if (!strncmp(iname,name,strlen(name)))  {
      XFree(iname);
      VERBOSE("found %s by iname %x \n",name,(unsigned int)top);
      return(top);
    }
    XFree(iname);
  }

  if(XGetClassHint(dpy,top,&xch))  {
   if(!strcmp(xch.res_class,name))  {
      XFree(xch.res_name); XFree(xch.res_class);
      VERBOSE("%s: res_class '%s' res_name '%s' %x \n",
             name,xch.res_class,xch.res_name,(unsigned int)top);
      return(top);
    }
    XFree(xch.res_name); XFree(xch.res_class);
  }

  if(!XQueryTree(dpy,top,&foo,&foo,&children,&nc) || children==NULL) {
    return(0);  // no more windows here
  }

  //check all the sub windows
  for(;nc>0;nc--)  {
    top = find_window(children[nc-1],name);
    if(top) break;  // we found it somewhere
  }
  if(children!=NULL) XFree(children);

  return(top);
}


int xerrhnd(Display *d, XErrorEvent *error)
{
	char msg[300];

	if(error->error_code==BadWindow) w=0;
	else {
		VERBOSE("Error Handler called:\n");
		XGetErrorText(error->display,(int)error->error_code,msg,300);
		VERBOSE("%s\n",msg);
	}
	return 0;
}

